@@ -13,13 +13,14 @@ import urllib.parse
 import aiohttp
 from .client_reqrep import ClientRequest, ClientResponse
 from .errors import WSServerHandshakeError
-from .multidict import MultiDictProxy, MultiDict, CIMultiDict
+from .multidict import MultiDictProxy, MultiDict, CIMultiDict, upstr
 from .websocket import WS_KEY, WebSocketParser, WebSocketWriter
 from .websocket_client import ClientWebSocketResponse
 from . import hdrs
 
 
-__all__ = ('request', 'ClientSession')
+__all__ = ('ClientSession', 'request', 'get', 'options', 'head',
+           'delete', 'post', 'put', 'patch')
 
 PY_341 = sys.version_info >= (3, 4, 1)
 
@@ -34,21 +35,24 @@ class ClientSession:
                  headers=None, auth=None, request_class=ClientRequest,
                  response_class=ClientResponse,
                  ws_response_class=ClientWebSocketResponse):
-        if loop is None:
-            loop = asyncio.get_event_loop()
+
+        if connector is None:
+            connector = aiohttp.TCPConnector(loop=loop)
+            loop = connector._loop  # never None
+        else:
+            if loop is None:
+                loop = connector._loop  # never None
+            elif connector._loop is not loop:
+                raise ValueError("loop argument must agree with connector")
+
         self._loop = loop
         if loop.get_debug():
             self._source_traceback = traceback.extract_stack(sys._getframe(1))
 
         self._cookies = http.cookies.SimpleCookie()
 
-        if connector is None:
-            connector = aiohttp.TCPConnector(loop=loop)
-        elif connector._loop is not loop:
-            raise ValueError("loop argument must agree with connector")
-
         # For Backward compatability with `share_cookies` connectors
-        elif connector._share_cookies:
+        if connector._share_cookies:
             self._update_cookies(connector.cookies)
         if cookies is not None:
             self._update_cookies(cookies)
@@ -101,7 +105,8 @@ class ClientSession:
             raise RuntimeError('Session is closed')
 
         redirects = 0
-        method = method.upper()
+        if not isinstance(method, upstr):
+            method = upstr(method)
 
         # Merge with default headers and transform to CIMultiDict
         headers = self._prepare_headers(headers)
@@ -129,7 +134,7 @@ class ClientSession:
                 try:
                     yield from resp.start(conn, read_until_eof)
                 except:
-                    resp.close()
+                    resp.close(force=True)
                     conn.close()
                     raise
             except (aiohttp.HttpProcessingError,
@@ -155,6 +160,8 @@ class ClientSession:
                 if resp.status != 307:
                     method = hdrs.METH_GET
                     data = None
+                    if headers.get(hdrs.CONTENT_LENGTH):
+                        headers.pop(hdrs.CONTENT_LENGTH)
 
                 r_url = (resp.headers.get(hdrs.LOCATION) or
                          resp.headers.get(hdrs.URI))
@@ -360,6 +367,12 @@ class ClientSession:
         """
         self._connector = None
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close()
+
 
 @asyncio.coroutine
 def request(method, url, *,
@@ -454,3 +467,45 @@ def request(method, url, *,
         return resp
     finally:
         session.detach()
+
+
+@asyncio.coroutine
+def get(url, **kwargs):
+    ret = yield from request(hdrs.METH_GET, url, **kwargs)
+    return ret
+
+
+@asyncio.coroutine
+def options(url, **kwargs):
+    ret = yield from request(hdrs.METH_OPTIONS, url, **kwargs)
+    return ret
+
+
+@asyncio.coroutine
+def head(url, **kwargs):
+    ret = yield from request(hdrs.METH_HEAD, url, **kwargs)
+    return ret
+
+
+@asyncio.coroutine
+def post(url, **kwargs):
+    ret = yield from request(hdrs.METH_POST, url, **kwargs)
+    return ret
+
+
+@asyncio.coroutine
+def put(url, **kwargs):
+    ret = yield from request(hdrs.METH_PUT, url, **kwargs)
+    return ret
+
+
+@asyncio.coroutine
+def patch(url, **kwargs):
+    ret = yield from request(hdrs.METH_PATCH, url, **kwargs)
+    return ret
+
+
+@asyncio.coroutine
+def delete(url, **kwargs):
+    ret = yield from request(hdrs.METH_DELETE, url, **kwargs)
+    return ret
