@@ -361,7 +361,11 @@ class DataQueue:
 
             assert not self._waiter
             self._waiter = asyncio.Future(loop=self._loop)
-            yield from self._waiter
+            try:
+                yield from self._waiter
+            except (asyncio.CancelledError, asyncio.TimeoutError):
+                self._waiter = None
+                raise
 
         if self._buffer:
             data, size = self._buffer.popleft()
@@ -389,6 +393,7 @@ class ChunksQueue(DataQueue):
 
 def maybe_resume(func):
 
+    @asyncio.coroutine
     @functools.wraps(func)
     def wrapper(self, *args, **kw):
         result = yield from func(self, *args, **kw)
