@@ -18,7 +18,7 @@ from aiohttp.connector import Connection
 
 from tests.test_client_functional import Functional
 
-PY_341 = sys.version_info >= (3, 4)
+PY_341 = sys.version_info >= (3, 4, 1)
 
 
 class TestHttpConnection(unittest.TestCase):
@@ -239,7 +239,7 @@ class TestBaseConnector(unittest.TestCase):
         tr, proto = unittest.mock.Mock(), unittest.mock.Mock()
         conn._conns[1] = [(tr, proto, self.loop.time() - 1000)]
         self.assertEqual(conn._get(1), (None, None))
-        self.assertEqual(conn._conns[1], [])
+        self.assertFalse(conn._conns)
         conn.close()
 
     def test_release(self):
@@ -253,7 +253,7 @@ class TestBaseConnector(unittest.TestCase):
 
         tr, proto = unittest.mock.Mock(), unittest.mock.Mock()
         key = 1
-        conn._acquired[key].append(tr)
+        conn._acquired[key].add(tr)
         conn._release(key, req, tr, proto)
         self.assertEqual(conn._conns[1][0], (tr, proto, 10))
         self.assertTrue(conn._start_cleanup_task.called)
@@ -273,13 +273,22 @@ class TestBaseConnector(unittest.TestCase):
 
         tr, proto = unittest.mock.Mock(), unittest.mock.Mock()
         key = 1
-        conn._acquired[key].append(tr)
+        conn._acquired[key].add(tr)
         conn._release(key, req, tr, proto)
         self.assertFalse(conn._conns)
         self.assertTrue(tr.close.called)
 
-    def test_release_pop_empty_conns(self):
-        # see issue #253
+    def test_get_pop_empty_conns(self):
+        # see issue #473
+        conn = aiohttp.BaseConnector(loop=self.loop)
+        key = ('127.0.0.1', 80, False)
+        conn._conns[key] = []
+        tr, proto = conn._get(key)
+        self.assertEqual((None, None), (tr, proto))
+        self.assertFalse(conn._conns)
+
+    def test_release_close_do_not_add_to_pool(self):
+        # see issue #473
         conn = aiohttp.BaseConnector(loop=self.loop)
         req = unittest.mock.Mock()
         resp = unittest.mock.Mock()
@@ -288,13 +297,10 @@ class TestBaseConnector(unittest.TestCase):
 
         key = ('127.0.0.1', 80, False)
 
-        conn._conns[key] = []
-
         tr, proto = unittest.mock.Mock(), unittest.mock.Mock()
-        conn._acquired[key].append(tr)
+        conn._acquired[key].add(tr)
         conn._release(key, req, tr, proto)
-        self.assertEqual({}, conn._conns)
-        self.assertTrue(tr.close.called)
+        self.assertFalse(conn._conns)
 
     def test_release_close_do_not_delete_existing_connections(self):
         key = ('127.0.0.1', 80, False)
@@ -309,7 +315,7 @@ class TestBaseConnector(unittest.TestCase):
         req.response = resp
 
         tr, proto = unittest.mock.Mock(), unittest.mock.Mock()
-        conn._acquired[key].append(tr)
+        conn._acquired[key].add(tr1)
         conn._release(key, req, tr, proto)
         self.assertEqual(conn._conns[key], [(tr1, proto1, 1)])
         self.assertTrue(tr.close.called)
@@ -324,7 +330,7 @@ class TestBaseConnector(unittest.TestCase):
 
         tr, proto = unittest.mock.Mock(), unittest.mock.Mock()
         key = 1
-        conn._acquired[key].append(tr)
+        conn._acquired[key].add(tr)
         conn._release(key, req, tr, proto)
         self.assertEqual(conn._conns, {1: [(tr, proto, 10)]})
         self.assertFalse(tr.close.called)
@@ -338,7 +344,7 @@ class TestBaseConnector(unittest.TestCase):
 
         tr, proto = unittest.mock.Mock(), unittest.mock.Mock()
         key = 1
-        conn._acquired[key].append(tr)
+        conn._acquired[key].add(tr)
         conn._release(key, req, tr, proto)
         self.assertTrue(tr.close.called)
 
@@ -458,8 +464,15 @@ class TestBaseConnector(unittest.TestCase):
         conn = aiohttp.TCPConnector(loop=self.loop)
         self.assertTrue(conn.verify_ssl)
         self.assertIs(conn.fingerprint, None)
-        self.assertFalse(conn.resolve)
+
+        with self.assertWarns(DeprecationWarning):
+            self.assertFalse(conn.resolve)
+        self.assertFalse(conn.use_dns_cache)
+
         self.assertEqual(conn.family, socket.AF_INET)
+
+        with self.assertWarns(DeprecationWarning):
+            self.assertEqual(conn.resolved_hosts, {})
         self.assertEqual(conn.resolved_hosts, {})
 
     def test_tcp_connector_ctor_fingerprint_valid(self):
@@ -514,17 +527,37 @@ class TestBaseConnector(unittest.TestCase):
     def test_tcp_connector_clear_resolved_hosts(self):
         conn = aiohttp.TCPConnector(loop=self.loop)
         info = object()
-        conn._resolved_hosts[('localhost', 123)] = info
-        conn._resolved_hosts[('localhost', 124)] = info
+        conn._cached_hosts[('localhost', 123)] = info
+        conn._cached_hosts[('localhost', 124)] = info
         conn.clear_resolved_hosts('localhost', 123)
         self.assertEqual(
             conn.resolved_hosts, {('localhost', 124): info})
         conn.clear_resolved_hosts('localhost', 123)
         self.assertEqual(
             conn.resolved_hosts, {('localhost', 124): info})
-        conn.clear_resolved_hosts()
+        with self.assertWarns(DeprecationWarning):
+            conn.clear_resolved_hosts()
         self.assertEqual(conn.resolved_hosts, {})
 
+    def test_tcp_connector_clear_dns_cache(self):
+        conn = aiohttp.TCPConnector(loop=self.loop)
+        info = object()
+        conn._cached_hosts[('localhost', 123)] = info
+        conn._cached_hosts[('localhost', 124)] = info
+        conn.clear_dns_cache('localhost', 123)
+        self.assertEqual(
+            conn.cached_hosts, {('localhost', 124): info})
+        conn.clear_dns_cache('localhost', 123)
+        self.assertEqual(
+            conn.cached_hosts, {('localhost', 124): info})
+        conn.clear_dns_cache()
+        self.assertEqual(conn.cached_hosts, {})
+
+    def test_tcp_connector_clear_dns_cache_bad_args(self):
+        conn = aiohttp.TCPConnector(loop=self.loop)
+        with self.assertRaises(ValueError):
+            conn.clear_dns_cache('localhost')
+
     def test_ambigous_verify_ssl_and_ssl_context(self):
         with self.assertRaises(ValueError):
             aiohttp.TCPConnector(
@@ -756,6 +789,25 @@ class TestHttpClientConnector(unittest.TestCase):
             conn = aiohttp.TCPConnector(share_cookies=True, loop=self.loop)
         conn.close()
 
+    def test_ambiguous_ctor_params(self):
+        with self.assertRaises(ValueError):
+            aiohttp.TCPConnector(resolve=True, use_dns_cache=False,
+                                 loop=self.loop)
+
+    def test_both_resolve_and_use_dns_cache(self):
+        conn = aiohttp.TCPConnector(resolve=True, use_dns_cache=True,
+                                    loop=self.loop)
+        self.assertTrue(conn.use_dns_cache)
+        with self.assertWarns(DeprecationWarning):
+            self.assertTrue(conn.resolve)
+
+    def test_both_use_dns_cache_only(self):
+        conn = aiohttp.TCPConnector(use_dns_cache=True,
+                                    loop=self.loop)
+        self.assertTrue(conn.use_dns_cache)
+        with self.assertWarns(DeprecationWarning):
+            self.assertTrue(conn.resolve)
+
 
 class TestProxyConnector(unittest.TestCase):
 
@@ -1092,3 +1144,42 @@ class TestProxyConnector(unittest.TestCase):
                                            loop=self.loop)
         self.assertIsNone(connector.proxy_auth)
         connector.close()
+
+    @unittest.mock.patch('aiohttp.connector.ClientRequest')
+    def test_https_connect_pass_ssl_context(self, ClientRequestMock):
+        loop_mock = unittest.mock.Mock()
+        proxy_req = ClientRequest('GET', 'http://proxy.example.com',
+                                  loop=loop_mock)
+        ClientRequestMock.return_value = proxy_req
+
+        proxy_resp = ClientResponse('get', 'http://proxy.example.com')
+        proxy_resp._loop = loop_mock
+        proxy_req.send = send_mock = unittest.mock.Mock()
+        send_mock.return_value = proxy_resp
+        proxy_resp.start = start_mock = unittest.mock.Mock()
+        self._fake_coroutine(start_mock, unittest.mock.Mock(status=200))
+
+        connector = aiohttp.ProxyConnector(
+            'http://proxy.example.com', loop=loop_mock)
+
+        tr, proto = unittest.mock.Mock(), unittest.mock.Mock()
+        self._fake_coroutine(loop_mock.create_connection, (tr, proto))
+
+        req = ClientRequest('GET', 'https://www.python.org', loop=self.loop)
+        self.loop.run_until_complete(connector._create_connection(req))
+
+        loop_mock.create_connection.assert_called_with(
+            mock.ANY,
+            ssl=connector.ssl_context,
+            sock=mock.ANY,
+            server_hostname='www.python.org')
+
+        self.assertEqual(req.path, '/')
+        self.assertEqual(proxy_req.method, 'CONNECT')
+        self.assertEqual(proxy_req.path, 'www.python.org:443')
+        tr.pause_reading.assert_called_once_with()
+        tr.get_extra_info.assert_called_with('socket', default=None)
+
+        proxy_req.close()
+        proxy_resp.close()
+        req.close()
