@@ -3,8 +3,6 @@ import unittest
 from unittest import mock
 
 from aiohttp import web, log
-from aiohttp.multidict import CIMultiDict
-from aiohttp.protocol import HttpVersion11, RawRequestMessage
 
 
 class TestWeb(unittest.TestCase):
@@ -16,21 +14,6 @@ class TestWeb(unittest.TestCase):
     def tearDown(self):
         self.loop.close()
 
-    def test_handler_returns_not_response(self):
-        app = web.Application(loop=self.loop)
-
-        def handler(request):
-            return 'abc'
-
-        app.router.add_route('GET', '/', handler)
-        h = app.make_handler()()
-        message = RawRequestMessage('GET', '/', HttpVersion11,
-                                    CIMultiDict(), False, False)
-        payload = mock.Mock()
-
-        with self.assertRaises(AssertionError):
-            self.loop.run_until_complete(h.handle_request(message, payload))
-
     def test_app_ctor(self):
         app = web.Application(loop=self.loop)
         self.assertIs(self.loop, app.loop)
@@ -143,7 +126,7 @@ class TestRequestHandlerFactory(unittest.TestCase):
 
         manager.connection_lost(handler, None)
         self.assertEqual(manager.connections, [])
-        handler.closing.assert_called_with()
+        handler.closing.assert_called_with(timeout=None)
         transport.close.assert_called_with()
 
     def test_finish_connection_timeout(self):
@@ -158,5 +141,5 @@ class TestRequestHandlerFactory(unittest.TestCase):
 
         manager.connection_lost(handler, None)
         self.assertEqual(manager.connections, [])
-        handler.closing.assert_called_with()
+        handler.closing.assert_called_with(timeout=0.09)
         transport.close.assert_called_with()
