@@ -3,7 +3,7 @@ import json
 import os.path
 import socket
 import unittest
-from aiohttp import log, web, request, FormData
+from aiohttp import log, web, request, FormData, ClientSession
 from aiohttp.multidict import MultiDict
 from aiohttp.protocol import HttpVersion, HttpVersion10, HttpVersion11
 from aiohttp.streams import EOF_MARKER
@@ -62,6 +62,20 @@ class TestWebFunctional(unittest.TestCase):
 
         self.loop.run_until_complete(go())
 
+    def test_handler_returns_not_response(self):
+
+        @asyncio.coroutine
+        def handler(request):
+            return 'abc'
+
+        @asyncio.coroutine
+        def go():
+            _, _, url = yield from self.create_server('GET', '/', handler)
+            resp = yield from request('GET', url, loop=self.loop)
+            self.assertEqual(500, resp.status)
+
+        self.loop.run_until_complete(go())
+
     def test_post_form(self):
 
         @asyncio.coroutine
@@ -181,6 +195,40 @@ class TestWebFunctional(unittest.TestCase):
 
         self.loop.run_until_complete(go())
 
+    def test_files_upload_with_same_key(self):
+        @asyncio.coroutine
+        def handler(request):
+            data = yield from request.post()
+            files = data.getall('file')
+            _file_names = []
+            for _file in files:
+                self.assertFalse(_file.file.closed)
+                if _file.filename == 'test1.jpeg':
+                    self.assertEqual(_file.file.read(), b'binary data 1')
+                if _file.filename == 'test2.jpeg':
+                    self.assertEqual(_file.file.read(), b'binary data 2')
+                _file_names.append(_file.filename)
+            self.assertCountEqual(_file_names, ['test1.jpeg', 'test2.jpeg'])
+            resp = web.Response(body=b'OK')
+            return resp
+
+        @asyncio.coroutine
+        def go():
+            _, _, url = yield from self.create_server('POST', '/', handler)
+            _data = FormData()
+            _data.add_field('file', b'binary data 1',
+                            content_type='image/jpeg',
+                            filename='test1.jpeg')
+            _data.add_field('file', b'binary data 2',
+                            content_type='image/jpeg',
+                            filename='test2.jpeg')
+            resp = yield from request('POST', url, data=_data,
+                                      loop=self.loop)
+            self.assertEqual(200, resp.status)
+            resp.close()
+
+        self.loop.run_until_complete(go())
+
     def test_post_files(self):
 
         here = os.path.dirname(__file__)
@@ -374,6 +422,87 @@ class TestWebFunctional(unittest.TestCase):
         filename = '../README.rst'
         self.loop.run_until_complete(go(here, filename))
 
+    def test_static_file_if_modified_since(self):
+
+        @asyncio.coroutine
+        def go(dirname, filename):
+            app, _, url = yield from self.create_server(
+                'GET', '/static/' + filename
+            )
+            app.router.add_static('/static', dirname)
+
+            resp = yield from request('GET', url, loop=self.loop)
+            self.assertEqual(200, resp.status)
+            lastmod = resp.headers.get('Last-Modified')
+            self.assertIsNotNone(lastmod)
+            resp.close()
+
+            resp = yield from request('GET', url, loop=self.loop,
+                                      headers={'If-Modified-Since': lastmod})
+            self.assertEqual(304, resp.status)
+            resp.close()
+
+        here = os.path.dirname(__file__)
+        filename = 'data.unknown_mime_type'
+        self.loop.run_until_complete(go(here, filename))
+
+    def test_static_file_if_modified_since_past_date(self):
+
+        @asyncio.coroutine
+        def go(dirname, filename):
+            app, _, url = yield from self.create_server(
+                'GET', '/static/' + filename
+            )
+            app.router.add_static('/static', dirname)
+
+            lastmod = 'Mon, 1 Jan 1990 01:01:01 GMT'
+            resp = yield from request('GET', url, loop=self.loop,
+                                      headers={'If-Modified-Since': lastmod})
+            self.assertEqual(200, resp.status)
+            resp.close()
+
+        here = os.path.dirname(__file__)
+        filename = 'data.unknown_mime_type'
+        self.loop.run_until_complete(go(here, filename))
+
+    def test_static_file_if_modified_since_future_date(self):
+
+        @asyncio.coroutine
+        def go(dirname, filename):
+            app, _, url = yield from self.create_server(
+                'GET', '/static/' + filename
+            )
+            app.router.add_static('/static', dirname)
+
+            lastmod = 'Fri, 31 Dec 9999 23:59:59 GMT'
+            resp = yield from request('GET', url, loop=self.loop,
+                                      headers={'If-Modified-Since': lastmod})
+            self.assertEqual(304, resp.status)
+            resp.close()
+
+        here = os.path.dirname(__file__)
+        filename = 'data.unknown_mime_type'
+        self.loop.run_until_complete(go(here, filename))
+
+    def test_static_file_if_modified_since_invalid_date(self):
+
+        @asyncio.coroutine
+        def go(dirname, filename):
+            app, _, url = yield from self.create_server(
+                'GET', '/static/' + filename
+            )
+            app.router.add_static('/static', dirname)
+
+            lastmod = 'not a valid HTTP-date'
+            resp = yield from request('GET', url, loop=self.loop,
+                                      headers={'If-Modified-Since': lastmod})
+            self.assertEqual(200, resp.status)
+            resp.close()
+
+        here = os.path.dirname(__file__)
+        filename = 'data.unknown_mime_type'
+        self.loop.run_until_complete(go(here, filename))
+
     def test_static_route_path_existence_check(self):
         directory = os.path.dirname(__file__)
         web.StaticRoute(None, "/", directory)
@@ -735,3 +864,60 @@ class TestWebFunctional(unittest.TestCase):
             self.assertEqual('data', txt)
 
         self.loop.run_until_complete(go())
+
+    def test_get_with_empty_arg(self):
+
+        @asyncio.coroutine
+        def handler(request):
+            self.assertIn('arg', request.GET)
+            self.assertEqual('', request.GET['arg'])
+            return web.Response()
+
+        @asyncio.coroutine
+        def go():
+            _, srv, url = yield from self.create_server('GET', '/', handler)
+            resp = yield from request('GET', url+'?arg', loop=self.loop)
+            self.assertEqual(200, resp.status)
+            yield from resp.release()
+
+        self.loop.run_until_complete(go())
+
+    def test_get_with_empty_arg_with_equal(self):
+
+        @asyncio.coroutine
+        def handler(request):
+            self.assertIn('arg', request.GET)
+            self.assertEqual('', request.GET['arg'])
+            return web.Response()
+
+        @asyncio.coroutine
+        def go():
+            _, srv, url = yield from self.create_server('GET', '/', handler)
+            resp = yield from request('GET', url+'?arg=', loop=self.loop)
+            self.assertEqual(200, resp.status)
+            yield from resp.release()
+
+        self.loop.run_until_complete(go())
+
+    def test_stream_response_multiple_chunks(self):
+        @asyncio.coroutine
+        def handler(request):
+            resp = web.StreamResponse()
+            resp.enable_chunked_encoding()
+            resp.start(request)
+            resp.write(b'x')
+            resp.write(b'y')
+            resp.write(b'z')
+            return resp
+
+        @asyncio.coroutine
+        def go():
+            _, srv, url = yield from self.create_server('GET', '/', handler)
+            client = ClientSession(loop=self.loop)
+            resp = yield from client.get(url)
+            self.assertEqual(200, resp.status)
+            data = yield from resp.read()
+            self.assertEqual(b'xyz', data)
+            yield from resp.release()
+
+        self.loop.run_until_complete(go())
