@@ -24,9 +24,9 @@ class TestWebSocketClient(unittest.TestCase):
     def tearDown(self):
         self.loop.close()
 
-    @mock.patch('aiohttp.websocket_client.os')
-    @mock.patch('aiohttp.websocket_client.client')
-    def test_ws_connect(self, m_client, m_os):
+    @mock.patch('aiohttp.client.os')
+    @mock.patch('aiohttp.client.ClientSession.request')
+    def test_ws_connect(self, m_req, m_os):
         resp = mock.Mock()
         resp.status = 101
         resp.headers = {
@@ -36,8 +36,8 @@ class TestWebSocketClient(unittest.TestCase):
             hdrs.SEC_WEBSOCKET_PROTOCOL: 'chat'
         }
         m_os.urandom.return_value = self.key_data
-        m_client.request.return_value = asyncio.Future(loop=self.loop)
-        m_client.request.return_value.set_result(resp)
+        m_req.return_value = asyncio.Future(loop=self.loop)
+        m_req.return_value.set_result(resp)
 
         res = self.loop.run_until_complete(
             websocket_client.ws_connect(
@@ -48,9 +48,36 @@ class TestWebSocketClient(unittest.TestCase):
         self.assertIsInstance(res, websocket_client.ClientWebSocketResponse)
         self.assertEqual(res.protocol, 'chat')
 
-    @mock.patch('aiohttp.websocket_client.os')
-    @mock.patch('aiohttp.websocket_client.client')
-    def test_ws_connect_global_loop(self, m_client, m_os):
+    @mock.patch('aiohttp.client.os')
+    @mock.patch('aiohttp.client.ClientSession.request')
+    def test_ws_connect_custom_response(self, m_req, m_os):
+
+        class CustomResponse(websocket_client.ClientWebSocketResponse):
+            def read(self, decode=False):
+                return 'customized!'
+
+        resp = mock.Mock()
+        resp.status = 101
+        resp.headers = {
+            hdrs.UPGRADE: hdrs.WEBSOCKET,
+            hdrs.CONNECTION: hdrs.UPGRADE,
+            hdrs.SEC_WEBSOCKET_ACCEPT: self.ws_key,
+        }
+        m_os.urandom.return_value = self.key_data
+        m_req.return_value = asyncio.Future(loop=self.loop)
+        m_req.return_value.set_result(resp)
+
+        res = self.loop.run_until_complete(
+            websocket_client.ws_connect(
+                'http://test.org',
+                ws_response_class=CustomResponse,
+                loop=self.loop))
+
+        self.assertEqual(res.read(), 'customized!')
+
+    @mock.patch('aiohttp.client.os')
+    @mock.patch('aiohttp.client.ClientSession.request')
+    def test_ws_connect_global_loop(self, m_req, m_os):
         asyncio.set_event_loop(self.loop)
 
         resp = mock.Mock()
@@ -61,8 +88,8 @@ class TestWebSocketClient(unittest.TestCase):
             hdrs.SEC_WEBSOCKET_ACCEPT: self.ws_key
         }
         m_os.urandom.return_value = self.key_data
-        m_client.request.return_value = asyncio.Future(loop=self.loop)
-        m_client.request.return_value.set_result(resp)
+        m_req.return_value = asyncio.Future(loop=self.loop)
+        m_req.return_value.set_result(resp)
 
         resp = self.loop.run_until_complete(
             websocket_client.ws_connect('http://test.org'))
@@ -70,9 +97,9 @@ class TestWebSocketClient(unittest.TestCase):
 
         asyncio.set_event_loop(None)
 
-    @mock.patch('aiohttp.websocket_client.os')
-    @mock.patch('aiohttp.websocket_client.client')
-    def test_ws_connect_err_status(self, m_client, m_os):
+    @mock.patch('aiohttp.client.os')
+    @mock.patch('aiohttp.client.ClientSession.request')
+    def test_ws_connect_err_status(self, m_req, m_os):
         resp = mock.Mock()
         resp.status = 500
         resp.headers = {
@@ -81,8 +108,8 @@ class TestWebSocketClient(unittest.TestCase):
             hdrs.SEC_WEBSOCKET_ACCEPT: self.ws_key
         }
         m_os.urandom.return_value = self.key_data
-        m_client.request.return_value = asyncio.Future(loop=self.loop)
-        m_client.request.return_value.set_result(resp)
+        m_req.return_value = asyncio.Future(loop=self.loop)
+        m_req.return_value.set_result(resp)
 
         with self.assertRaises(errors.WSServerHandshakeError) as ctx:
             self.loop.run_until_complete(
@@ -93,9 +120,9 @@ class TestWebSocketClient(unittest.TestCase):
         self.assertEqual(
             ctx.exception.message, 'Invalid response status')
 
-    @mock.patch('aiohttp.websocket_client.os')
-    @mock.patch('aiohttp.websocket_client.client')
-    def test_ws_connect_err_upgrade(self, m_client, m_os):
+    @mock.patch('aiohttp.client.os')
+    @mock.patch('aiohttp.client.ClientSession.request')
+    def test_ws_connect_err_upgrade(self, m_req, m_os):
         resp = mock.Mock()
         resp.status = 101
         resp.headers = {
@@ -104,8 +131,8 @@ class TestWebSocketClient(unittest.TestCase):
             hdrs.SEC_WEBSOCKET_ACCEPT: self.ws_key
         }
         m_os.urandom.return_value = self.key_data
-        m_client.request.return_value = asyncio.Future(loop=self.loop)
-        m_client.request.return_value.set_result(resp)
+        m_req.return_value = asyncio.Future(loop=self.loop)
+        m_req.return_value.set_result(resp)
 
         with self.assertRaises(errors.WSServerHandshakeError) as ctx:
             self.loop.run_until_complete(
@@ -116,9 +143,9 @@ class TestWebSocketClient(unittest.TestCase):
         self.assertEqual(
             ctx.exception.message, 'Invalid upgrade header')
 
-    @mock.patch('aiohttp.websocket_client.os')
-    @mock.patch('aiohttp.websocket_client.client')
-    def test_ws_connect_err_conn(self, m_client, m_os):
+    @mock.patch('aiohttp.client.os')
+    @mock.patch('aiohttp.client.ClientSession.request')
+    def test_ws_connect_err_conn(self, m_req, m_os):
         resp = mock.Mock()
         resp.status = 101
         resp.headers = {
@@ -127,8 +154,8 @@ class TestWebSocketClient(unittest.TestCase):
             hdrs.SEC_WEBSOCKET_ACCEPT: self.ws_key
         }
         m_os.urandom.return_value = self.key_data
-        m_client.request.return_value = asyncio.Future(loop=self.loop)
-        m_client.request.return_value.set_result(resp)
+        m_req.return_value = asyncio.Future(loop=self.loop)
+        m_req.return_value.set_result(resp)
 
         with self.assertRaises(errors.WSServerHandshakeError) as ctx:
             self.loop.run_until_complete(
@@ -139,9 +166,9 @@ class TestWebSocketClient(unittest.TestCase):
         self.assertEqual(
             ctx.exception.message, 'Invalid connection header')
 
-    @mock.patch('aiohttp.websocket_client.os')
-    @mock.patch('aiohttp.websocket_client.client')
-    def test_ws_connect_err_challenge(self, m_client, m_os):
+    @mock.patch('aiohttp.client.os')
+    @mock.patch('aiohttp.client.ClientSession.request')
+    def test_ws_connect_err_challenge(self, m_req, m_os):
         resp = mock.Mock()
         resp.status = 101
         resp.headers = {
@@ -150,8 +177,8 @@ class TestWebSocketClient(unittest.TestCase):
             hdrs.SEC_WEBSOCKET_ACCEPT: 'asdfasdfasdfasdfasdfasdf'
         }
         m_os.urandom.return_value = self.key_data
-        m_client.request.return_value = asyncio.Future(loop=self.loop)
-        m_client.request.return_value.set_result(resp)
+        m_req.return_value = asyncio.Future(loop=self.loop)
+        m_req.return_value.set_result(resp)
 
         with self.assertRaises(errors.WSServerHandshakeError) as ctx:
             self.loop.run_until_complete(
@@ -162,10 +189,10 @@ class TestWebSocketClient(unittest.TestCase):
         self.assertEqual(
             ctx.exception.message, 'Invalid challenge response')
 
-    @mock.patch('aiohttp.websocket_client.WebSocketWriter')
-    @mock.patch('aiohttp.websocket_client.os')
-    @mock.patch('aiohttp.websocket_client.client')
-    def test_close(self, m_client, m_os, WebSocketWriter):
+    @mock.patch('aiohttp.client.WebSocketWriter')
+    @mock.patch('aiohttp.client.os')
+    @mock.patch('aiohttp.client.ClientSession.request')
+    def test_close(self, m_req, m_os, WebSocketWriter):
         resp = mock.Mock()
         resp.status = 101
         resp.headers = {
@@ -174,8 +201,8 @@ class TestWebSocketClient(unittest.TestCase):
             hdrs.SEC_WEBSOCKET_ACCEPT: self.ws_key,
         }
         m_os.urandom.return_value = self.key_data
-        m_client.request.return_value = asyncio.Future(loop=self.loop)
-        m_client.request.return_value.set_result(resp)
+        m_req.return_value = asyncio.Future(loop=self.loop)
+        m_req.return_value.set_result(resp)
         writer = WebSocketWriter.return_value = mock.Mock()
         reader = resp.connection.reader.set_parser.return_value = mock.Mock()
 
@@ -198,10 +225,10 @@ class TestWebSocketClient(unittest.TestCase):
         self.assertFalse(res)
         self.assertEqual(writer.close.call_count, 1)
 
-    @mock.patch('aiohttp.websocket_client.WebSocketWriter')
-    @mock.patch('aiohttp.websocket_client.os')
-    @mock.patch('aiohttp.websocket_client.client')
-    def test_close_exc(self, m_client, m_os, WebSocketWriter):
+    @mock.patch('aiohttp.client.WebSocketWriter')
+    @mock.patch('aiohttp.client.os')
+    @mock.patch('aiohttp.client.ClientSession.request')
+    def test_close_exc(self, m_req, m_os, WebSocketWriter):
         resp = mock.Mock()
         resp.status = 101
         resp.headers = {
@@ -210,8 +237,8 @@ class TestWebSocketClient(unittest.TestCase):
             hdrs.SEC_WEBSOCKET_ACCEPT: self.ws_key,
         }
         m_os.urandom.return_value = self.key_data
-        m_client.request.return_value = asyncio.Future(loop=self.loop)
-        m_client.request.return_value.set_result(resp)
+        m_req.return_value = asyncio.Future(loop=self.loop)
+        m_req.return_value.set_result(resp)
         WebSocketWriter.return_value = mock.Mock()
         reader = resp.connection.reader.set_parser.return_value = mock.Mock()
 
@@ -228,10 +255,10 @@ class TestWebSocketClient(unittest.TestCase):
         self.assertTrue(resp.closed)
         self.assertIs(resp.exception(), exc)
 
-    @mock.patch('aiohttp.websocket_client.WebSocketWriter')
-    @mock.patch('aiohttp.websocket_client.os')
-    @mock.patch('aiohttp.websocket_client.client')
-    def test_close_exc2(self, m_client, m_os, WebSocketWriter):
+    @mock.patch('aiohttp.client.WebSocketWriter')
+    @mock.patch('aiohttp.client.os')
+    @mock.patch('aiohttp.client.ClientSession.request')
+    def test_close_exc2(self, m_req, m_os, WebSocketWriter):
         resp = mock.Mock()
         resp.status = 101
         resp.headers = {
@@ -240,8 +267,8 @@ class TestWebSocketClient(unittest.TestCase):
             hdrs.SEC_WEBSOCKET_ACCEPT: self.ws_key,
         }
         m_os.urandom.return_value = self.key_data
-        m_client.request.return_value = asyncio.Future(loop=self.loop)
-        m_client.request.return_value.set_result(resp)
+        m_req.return_value = asyncio.Future(loop=self.loop)
+        m_req.return_value.set_result(resp)
         writer = WebSocketWriter.return_value = mock.Mock()
         resp.connection.reader.set_parser.return_value = mock.Mock()
 
@@ -262,10 +289,10 @@ class TestWebSocketClient(unittest.TestCase):
         self.assertRaises(asyncio.CancelledError,
                           self.loop.run_until_complete, resp.close())
 
-    @mock.patch('aiohttp.websocket_client.WebSocketWriter')
-    @mock.patch('aiohttp.websocket_client.os')
-    @mock.patch('aiohttp.websocket_client.client')
-    def test_send_data_after_close(self, m_client, m_os, WebSocketWriter):
+    @mock.patch('aiohttp.client.WebSocketWriter')
+    @mock.patch('aiohttp.client.os')
+    @mock.patch('aiohttp.client.ClientSession.request')
+    def test_send_data_after_close(self, m_req, m_os, WebSocketWriter):
         resp = mock.Mock()
         resp.status = 101
         resp.headers = {
@@ -274,8 +301,8 @@ class TestWebSocketClient(unittest.TestCase):
             hdrs.SEC_WEBSOCKET_ACCEPT: self.ws_key,
         }
         m_os.urandom.return_value = self.key_data
-        m_client.request.return_value = asyncio.Future(loop=self.loop)
-        m_client.request.return_value.set_result(resp)
+        m_req.return_value = asyncio.Future(loop=self.loop)
+        m_req.return_value.set_result(resp)
         WebSocketWriter.return_value = mock.Mock()
 
         resp = self.loop.run_until_complete(
@@ -288,10 +315,10 @@ class TestWebSocketClient(unittest.TestCase):
         self.assertRaises(RuntimeError, resp.send_str, 's')
         self.assertRaises(RuntimeError, resp.send_bytes, b'b')
 
-    @mock.patch('aiohttp.websocket_client.WebSocketWriter')
-    @mock.patch('aiohttp.websocket_client.os')
-    @mock.patch('aiohttp.websocket_client.client')
-    def test_send_data_type_errors(self, m_client, m_os, WebSocketWriter):
+    @mock.patch('aiohttp.client.WebSocketWriter')
+    @mock.patch('aiohttp.client.os')
+    @mock.patch('aiohttp.client.ClientSession.request')
+    def test_send_data_type_errors(self, m_req, m_os, WebSocketWriter):
         resp = mock.Mock()
         resp.status = 101
         resp.headers = {
@@ -300,8 +327,8 @@ class TestWebSocketClient(unittest.TestCase):
             hdrs.SEC_WEBSOCKET_ACCEPT: self.ws_key,
         }
         m_os.urandom.return_value = self.key_data
-        m_client.request.return_value = asyncio.Future(loop=self.loop)
-        m_client.request.return_value.set_result(resp)
+        m_req.return_value = asyncio.Future(loop=self.loop)
+        m_req.return_value.set_result(resp)
         WebSocketWriter.return_value = mock.Mock()
 
         resp = self.loop.run_until_complete(
@@ -311,10 +338,10 @@ class TestWebSocketClient(unittest.TestCase):
         self.assertRaises(TypeError, resp.send_str, b's')
         self.assertRaises(TypeError, resp.send_bytes, 'b')
 
-    @mock.patch('aiohttp.websocket_client.WebSocketWriter')
-    @mock.patch('aiohttp.websocket_client.os')
-    @mock.patch('aiohttp.websocket_client.client')
-    def test_reader_read_exception(self, m_client, m_os, WebSocketWriter):
+    @mock.patch('aiohttp.client.WebSocketWriter')
+    @mock.patch('aiohttp.client.os')
+    @mock.patch('aiohttp.client.ClientSession.request')
+    def test_reader_read_exception(self, m_req, m_os, WebSocketWriter):
         hresp = mock.Mock()
         hresp.status = 101
         hresp.headers = {
@@ -323,8 +350,8 @@ class TestWebSocketClient(unittest.TestCase):
             hdrs.SEC_WEBSOCKET_ACCEPT: self.ws_key,
         }
         m_os.urandom.return_value = self.key_data
-        m_client.request.return_value = asyncio.Future(loop=self.loop)
-        m_client.request.return_value.set_result(hresp)
+        m_req.return_value = asyncio.Future(loop=self.loop)
+        m_req.return_value.set_result(hresp)
         WebSocketWriter.return_value = mock.Mock()
         reader = hresp.connection.reader.set_parser.return_value = mock.Mock()
 
